@@ -13,29 +13,26 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import sys
 import errno
 import hashlib
-
-from libcloud.common.exceptions import RateLimitReachedError
-from libcloud.utils.py3 import httplib
+import sys
 from io import BytesIO
 
 import mock
 from mock import Mock
 
-from libcloud.utils.py3 import StringIO
-from libcloud.utils.py3 import b
-from libcloud.utils.py3 import PY2
-from libcloud.utils.py3 import assertRaisesRegex
-
-from libcloud.storage.base import StorageDriver
+from libcloud.common.exceptions import RateLimitReachedError
 from libcloud.storage.base import DEFAULT_CONTENT_TYPE
-
-from libcloud.test import unittest
-from libcloud.test import MockHttp
+from libcloud.storage.base import StorageDriver
 from libcloud.test import BodyStream
+from libcloud.test import MockHttp
+from libcloud.test import unittest
 from libcloud.test.storage.base import BaseRangeDownloadMockHttp
+from libcloud.utils.py3 import PY2
+from libcloud.utils.py3 import StringIO
+from libcloud.utils.py3 import assertRaisesRegex
+from libcloud.utils.py3 import b
+from libcloud.utils.py3 import httplib
 
 
 class BaseMockRawResponse(MockHttp):
@@ -312,6 +309,36 @@ class BaseStorageTests(unittest.TestCase):
                 stream=iter([]),
             )
 
+    @mock.patch("os.environ", {"LIBCLOUD_RETRY_FAILED_HTTP_REQUESTS": True})
+    def test_should_retry_rate_limited_errors_until_success(self):
+        count = 0
+
+        def succeed_on_second(*_, **__) -> mock.MagicMock:
+            nonlocal count
+            count += 1
+            if count > 1:
+                successful_response = mock.MagicMock()
+                successful_response.status_code = 200
+                return successful_response
+            else:
+                raise RateLimitReachedError()
+
+        self.driver1.connection.connection.session.send = Mock(
+            side_effect=succeed_on_second
+        )
+        uploaded_object = self.driver1._upload_object(
+            object_name="some name",
+            content_type="something",
+            request_path="/",
+            stream=iter([]),
+        )
+
+        self.assertEqual(
+            True,
+            uploaded_object["response"].success(),
+            "Expected to have successful response after retry",
+        )
+
 
 class BaseRangeDownloadMockHttpTestCase(unittest.TestCase):
     def test_get_start_and_end_bytes_from_range_str(self):
